import { logger } from '@crosslab/service-common';
import { Mutex } from 'async-mutex';
import Queue from 'queue';
class SimpleQueueManager {
    mutex = new Mutex();
    queueMap = new Map();
    createQueue(id) {
        const queue = new Queue({
            concurrency: 1,
            autostart: true,
        });
        queue.addEventListener('error', error => {
            if (error)
                logger.log('error', `An error occurred while processing a job in the queue for ${id}`);
            else {
                this.mutex.acquire().then(internalRelease => {
                    try {
                        if (queue.length === 0)
                            this.queueMap.delete(id);
                    }
                    finally {
                        internalRelease();
                    }
                });
            }
        });
        return queue;
    }
    async addToQueue(id, handler) {
        const release = await this.mutex.acquire();
        try {
            const queue = this.queueMap.get(id) ?? this.createQueue(id);
            if (!this.queueMap.has(id))
                this.queueMap.set(id, queue);
            queue.push(async () => {
                await handler();
            });
        }
        finally {
            release();
        }
    }
}
export const simpleQueueManager = new SimpleQueueManager();
//# sourceMappingURL=queueManager.js.map